package io.github.wulkanowy.data.repositories.timetable

import androidx.room.Room
import androidx.test.core.app.ApplicationProvider
import androidx.test.ext.junit.runners.AndroidJUnit4
import io.github.wulkanowy.data.db.AppDatabase
import io.github.wulkanowy.data.db.entities.Semester
import io.github.wulkanowy.data.db.entities.Timetable
import org.junit.After
import org.junit.Before
import org.junit.Test
import org.junit.runner.RunWith
import org.threeten.bp.LocalDate
import org.threeten.bp.LocalDateTime
import kotlin.test.assertEquals

@RunWith(AndroidJUnit4::class)
class TimetableLocalTest {

    private lateinit var timetableDb: TimetableLocal

    private lateinit var testDb: AppDatabase

    @Before
    fun createDb() {
        testDb = Room.inMemoryDatabaseBuilder(ApplicationProvider.getApplicationContext(), AppDatabase::class.java).build()
        timetableDb = TimetableLocal(testDb.timetableDao)
    }

    @After
    fun closeDb() {
        testDb.close()
    }

    @Test
    fun saveAndReadTest() {
        timetableDb.saveTimetable(listOf(
                Timetable(1, 2, 1, LocalDateTime.now(), LocalDateTime.now(),
                        LocalDate.of(2018, 9, 10), "", "", "", "", "", false, false),
                Timetable(1, 2, 1, LocalDateTime.now(), LocalDateTime.now(),
                        LocalDate.of(2018, 9, 14), "", "", "", "", "", false, false),
                Timetable(1, 2, 1, LocalDateTime.now(), LocalDateTime.now(),
                        LocalDate.of(2018, 9, 17), "", "", "", "", "", false, false)
        ))

        val exams = timetableDb.getTimetable(
            Semester(1, 2, "", 1, 1, true, 1, 1),
                LocalDate.of(2018, 9, 10),
                LocalDate.of(2018, 9, 14)
        ).blockingGet()
        assertEquals(2, exams.size)
        assertEquals(exams[0].date, LocalDate.of(2018, 9, 10))
        assertEquals(exams[1].date, LocalDate.of(2018, 9, 14))
    }
}
